;;;; lastseen.scm - Display when a user was last seen when asked -*- scheme -*-
;;;; Copyright (C) 2002  Neal H. Walfield  <neal@cs.uml.edu>
;;;;
;;;; This file is part of Deride
;;;;
;;;; Deride is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2, or (at your option)
;;;; any later version.
;;;; 
;;;; Deride is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with the Deride; see the file COPYING.  If not, write to
;;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

(define-module (backend lastseen))

(use-modules (ice-9 regex))
(use-modules (comm hooks))
(use-modules (comm send))
(use-modules (conf deride))
(use-modules (forum hooks))
(use-modules (forum users))

;; Map of names to lists of file-ports.  Keys should compare equal
;; using `eqv?'.
(define-public last-seen-list '())

;; Get the data associated with a user name.
(define-public (get-last-seen user)
  (assoc-ref last-seen-list (string-downcase user)))

;; Set the data associated with a user name.
(define-public (set-last-seen user list)
  (set! last-seen-list
	(assoc-set! last-seen-list (string-downcase user) list)))

;; Get the user data for a forum
(define-public (get-user-last-seen-in-forum user forum)
  (let ((user-list (get-last-seen user)))
    (if user-list
	(assoc-ref user-list forum)
	#f)))

;; Set one data for a user
(define-public (set-user-last-seen-in-forum user forum data)
  (let* ((forum (string-downcase forum))
	 (user-list (get-last-seen user))
	 (list (assoc-set! user-list forum data)))
    (set-last-seen user list)))

;; The format of a seen query.
(define seen-re (make-regexp
		 (string-append
		  "^ *"
		  "(" deride-nick " *[:, ] *)?"
		  "seen +([a-zA-Z0-9_-]+)")
		 regexp/icase))

;; Return a random element from a list.
(define (random-element-from-list list)
  (list-ref list (random (length list))))

(define (tell-last-seen-to-forum forum asker user)
  (let ((last-saw (get-user-last-seen-in-forum user forum))
	(now (current-time)))
    (cond
     ; Asking about himself.
     ((string-ci=? asker user)
      (send-to-forum forum
		     (string-append
		      asker ": "
		      (random-element-from-list
		       '("Try looking in the mirror."
			 "Never."
			 "Maybe you need to ask yourself that question."
			 "Feeling okay?")))))

     ; Asking about me?
     ((string-ci=? deride-nick user)
      (send-to-forum forum
		     (string-append
		      asker ": "
		      (random-element-from-list
		       '("Everyday."
			 "I cannot get enough of that beautiful face!"
			 "Whenever I get a chance."
			 "Right in front of you; and isn't he sexy!")))))

     ; Is in the forum, however, has never spoken.
     ((and (not last-saw)
	   (forum-user-is-present forum user))
      (send-to-forum forum
		     (random-element-from-list
		      (list
		       (string-append
			asker ": " user " is quietly lurking.")
		       (string-append
			asker ": " user " has been lurking.")
		       (string-append
			asker ": Apparently, " user
			" is not the talkative type.")))))

     ; Never seen.
     ((not last-saw)
      (send-to-forum forum
		     (string-append
		      asker ": " 
		      (random-element-from-list
		       (list "nope."
			     "not recently."
			     (string-append "I have not seen " user
					    " online for a while"))))))

     ; Seen, but has left
     ((not (forum-user-is-present forum user))
      (let* ((dt (- now last-saw)))
	(cond
	 ((< dt 60) "just left.")
	 ((< dt (* 15 60))
	  (string-append "left "
			 (number->string (inexact->extact (/ dt 60)))
			 " minutes ago."))
	 ((< dt (* 2 60 60))
	  (string-append "was here "
			 (number->string (inexact->extact (/ dt (* 60 60))))
			 " hours ago."))
	 (string-append "was last on "
			(number->string (inexact->extact (/ dt (* 24 60 60))))
			" days ago."))))

     ; Said something less than a minute ago.
     ((< (- now last-saw) (* 1 60))
      (send-to-forum forum
		     (string-append
		      asker ": " 
		      (random-element-from-list
		       '("Look around."
			 "Open your eyes."
			 "Umm, right next to you.")))))

     ; Said something less than 10 minutes ago.
     ((< (- now last-saw) (* 15 60))
      (send-to-forum forum
		     (string-append
		      asker ": " user
		      (random-element-from-list
		       '(" has been chatting off and on."
			 " was talking just a fe minutes ago.")))))

     ; Said something less than an hour ago.
     ((< (- now last-saw) (* 60 60))
      (send-to-forum forum
		     (string-append
		      asker ": " 
		      (random-element-from-list
		       (list
			(string-append user " must have little while ago.")
			(string-append user " just took off.")
			(string-append user " was here about "
				       (number->string
					(* 5 (inexact->exact
					      (/ (- now last-saw)
						 (* 5 60)))))
					" minutes ago."))))))

     ; In the last day
     ((< (- now last-saw) (* 24 60 60))
      (let ((t (* 60 (inexact->exact (/ (- now last-saw) (* 60 60))))))
	(send-to-forum forum
		       (string-append asker ": I have not seen "
				      user " for "
				      (if (= t 1)
					  "over an hour."
					  (string-append
					   (number->string t)
					   " hours."))))))

     ; For at least a day.
     (let ((t (* 24 60 (inexact->exact (/ (- now last-saw)
					  (* 24 60 60))))))
       (send-to-forum forum
		      (string-append asker ": " user " has been idle for "
				     (if (= t 1)
					 "over a day."
					 (string-append
					  (number->string t)
					  " days."))))))))

(define (tell-last-seen-to-user asker user)
  (let ((list (get-last-seen user)))
    (if (not list)
	(send-to-user asker (string-append user " has never spoken."))
	(letrec
	    ((f (lambda (l)
		  (if (not l)
		      '()
		      (let* ((forum (caar l))
			     (last-saw (cdar l))
			     (dt (- (current-time) last-saw))
			     (days (inexact->exact (/ dt (* 24 60 60))))
			     (dt (- dt (* days 24 60 60)))
			     (hours (inexact->exact (/ dt (* 60 60))))
			     (dt (- dt (* hours (* 60 60))))
			     (minutes (inexact->exact (/ dt 60))))
			(send-to-user
			 asker
			 (string-append user " last spoke on "
					forum " " 
					(if (> days 1)
					    (string-append
					     (number->string days)
					     " days, ")
					    "")
					(if (= days 1)
					    "1 day, "
					    "")
					(if (> hours 1)
					    (string-append
					     (number->string hours)
					     " hours, ")
					    "")
					(if (= hours 1)
					    "1 hours, "
					    "")
					(number->string minutes)
					(if (= minutes 1)
					    " minute"
					    " minutes")
					" ago."))
			(f (cdr l)))))))
	  (f (get-last-seen user))))))

(add-hook! comm-received-forum-message-hook
	   (lambda (sender forum say msg)
	     (if (eq? (string-ref forum 0) #\#)
		 (begin
		   (let ((match (regexp-exec seen-re msg)))
		     (if match
			 (tell-last-seen-to-forum
			  forum sender (match:substring match 2))))
		   (set-user-last-seen-in-forum sender forum
						(current-time))))))

(add-hook! comm-received-personal-message-hook
	   (lambda (sender say msg)
	     (let ((match (regexp-exec seen-re msg)))
	       (if match
		   (tell-last-seen-to-user sender
					   (match:substring match 2))))))


; (add-hook! comm-entered-forum
;	   (lambda (forum users)

; (add-hook! forum-user-joined-hook
; 	   (lambda (user forum reason)

; (add-hook! forum-user-parted-hook
; 	   (lambda (user forum reason)

; (add-hook! forum-user-quit-hook
; 	   (lambda (user forum-list reason)

; (add-hook! forum-user-schizophrenia-hook
; 	   (lambda (old new forum-list reason) 
